from django.conf import settings as django_settings
from django_common.session import SessionManager


def common_settings(request):
    return {
        'domain_name': django_settings.DOMAIN_NAME,
        'www_root': django_settings.WWW_ROOT,
        'is_dev': django_settings.IS_DEV,
        'is_prod': django_settings.IS_PROD,
        'usertime': SessionManager(request).get_usertime()
    }



from django.core.management.base import BaseCommand

from optparse import make_option

from django_common.scaffold import Scaffold
from django_common import settings


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--model', default=None, dest='model', \
            help="""model name - only one model name per run is allowed. \n
            It requires additional fields parameters:

            char - CharField \t\t\t\t
            text - TextField \t\t\t\t
            int - IntegerFIeld \t\t\t\t
            decimal -DecimalField \t\t\t\t
            datetime - DateTimeField \t\t\t\t
            foreign - ForeignKey \t\t\t\t

            Example usages: \t\t\t\t

                --model forum char:title  text:body int:posts datetime:create_date \t\t
                --model blog foreign:blog:Blog, foreign:post:Post, foreign:added_by:User \t\t
                --model finance decimal:total_cost:10:2

            """),
    )

    def handle(self, *args, **options):
        if len(args) == 0:
            print "You must provide app name. For example:\n\npython manage.py scallfold my_app\n"
            return
        scaffold = Scaffold(args[0], options['model'], args)
        scaffold.run()

    def get_version(self):
        return 'django-common version: %s' % settings.VERSION



import binascii
import random
import string

from django.db.models import fields
from django.template.defaultfilters import slugify
from django.db import models
from django.core.serializers.json import DjangoJSONEncoder
from django.utils import simplejson as json
from django import forms
from django.conf import settings


from south.modelsinspector import add_introspection_rules
from django_common.helper import md5_hash


class JSONField(models.TextField):
    """
    JSONField is a generic textfield that neatly serializes/unserializes JSON objects seamlessly
    """

    # Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""

        if value == "":
            return None

        try:
            if isinstance(value, basestring):
                return json.loads(value)
        except ValueError:
            pass

        return value

    def get_db_prep_save(self, value, connection=None):
        """Convert our JSON object to a string before we save"""

        if value == "":
            return None

        if isinstance(value, dict):
            value = json.dumps(value, cls=DjangoJSONEncoder)

        return super(JSONField, self).get_db_prep_save(value, connection)


class UniqueSlugField(fields.SlugField):
    """
    Represents a self-managing sluf field, that makes sure that the slug value is unique on the db table. Slugs by
    default get a db_index on them. The "Unique" in the class name is a misnomer since it does support unique=False

    @requires "prepopulate_from" in the constructor. This could be a field or a function in the model class which is using
    this field

    Defaults update_on_save to False

    Taken and edited from: http://www.djangosnippets.org/snippets/728/
    """
    def __init__(self, prepopulate_from='id', *args, **kwargs):
        if kwargs.get('update_on_save'):
            self.__update_on_save = kwargs.pop('update_on_save')
        else:
            self.__update_on_save = False
        self.prepopulate_from = prepopulate_from
        super(UniqueSlugField, self).__init__(*args, **kwargs)

    def pre_save(self, model_instance, add):
        prepopulate_field = getattr(model_instance, self.prepopulate_from)
        if callable(prepopulate_field):
            prepopulate_value = prepopulate_field()
        else:
            prepopulate_value = prepopulate_field

        # if object has an id, and not to update on save, then return existig model instance's slug value
        if getattr(model_instance, 'id') and not self.__update_on_save:
            return getattr(model_instance, self.name)

        # if this is a previously saved object, and current instance's slug is same as one being proposed
        if getattr(model_instance, 'id') and getattr(model_instance, self.name) == slugify(prepopulate_value):
            return getattr(model_instance, self.name)

        # if a unique slug is not required (not the default of course)
        if not self.unique:
            return self.__set_and_return(model_instance, self.name, slugify(prepopulate_value))

        return self.__unique_slug(model_instance.__class__, model_instance, self.name,
                                  prepopulate_value)

    def __unique_slug(self, model, model_instance, slug_field, slug_value):
        orig_slug = slug = slugify(slug_value)
        index = 1
        while True:
            try:
                model.objects.get(**{slug_field: slug})
                index += 1
                slug = orig_slug + '-' + str(index)
            except model.DoesNotExist:
                return self.__set_and_return(model_instance, slug_field, slug)

    def __set_and_return(self, model_instance, slug_field, slug):
        setattr(model_instance, slug_field, slug)
        return slug

add_introspection_rules([
    (
        [UniqueSlugField],  # Class(es) these apply to
        [],         # Positional arguments (not used)
        {           # Keyword argument
            "prepopulate_from": ["prepopulate_from", {"default": 'id'}],
        },
    ),
], ["^django_common\.db_fields\.UniqueSlugField"])


class RandomHashField(fields.CharField):
    """
    Store a random hash for a certain model field.

    @param update_on_save optional field whether to update this hash or not, everytime the model instance is saved
    """
    def __init__(self, update_on_save=False, hash_length=None, *args, **kwargs):
        #TODO: args & kwargs serve no purpose but to make django evolution to work
        self.update_on_save = update_on_save
        self.hash_length = hash_length
        super(fields.CharField, self).__init__(max_length=128, unique=True, blank=False, null=False, db_index=True, default=md5_hash(max_length=self.hash_length))

    def pre_save(self, model_instance, add):
        if not add and not self.update_on_save:
            return getattr(model_instance, self.name)

        random_hash = md5_hash(max_length=self.hash_length)
        setattr(model_instance, self.name, random_hash)
        return random_hash

add_introspection_rules([
    (
        [RandomHashField],  # Class(es) these apply to
        [],         # Positional arguments (not used)
        {           # Keyword argument
            "update_on_save": ["update_on_save", {"default": False}],
            "hash_length": ["hash_length", {"default": None}],
        },
    ),
], ["^django_common\.db_fields\.RandomHashField"])


class BaseEncryptedField(models.Field):
    '''This code is based on the djangosnippet #1095
    You can find the original at http://www.djangosnippets.org/snippets/1095/'''

    def __init__(self, *args, **kwargs):
        cipher = kwargs.pop('cipher', 'AES')
        imp = __import__('Crypto.Cipher', globals(), locals(), [cipher], -1)
        self.cipher = getattr(imp, cipher).new(settings.SECRET_KEY[:32])
        self.prefix = '$%s$' % cipher

        max_length = kwargs.get('max_length', 40)
        mod = max_length % self.cipher.block_size
        if mod > 0:
            max_length += self.cipher.block_size - mod
        kwargs['max_length'] = max_length * 2 + len(self.prefix)

        models.Field.__init__(self, *args, **kwargs)

    def _is_encrypted(self, value):
        return isinstance(value, basestring) and value.startswith(self.prefix)

    def _get_padding(self, value):
        mod = len(value) % self.cipher.block_size
        if mod > 0:
            return self.cipher.block_size - mod
        return 0

    def to_python(self, value):
        if self._is_encrypted(value):
            return self.cipher.decrypt(binascii.a2b_hex(value[len(self.prefix):])).split('\0')[0]
        return value

    def get_db_prep_value(self, value, connection=None, prepared=None):
        if value is not None and not self._is_encrypted(value):
            padding = self._get_padding(value)
            if padding > 0:
                value += "\0" + ''.join([random.choice(string.printable) for index in range(padding - 1)])
            value = self.prefix + binascii.b2a_hex(self.cipher.encrypt(value))
        return value


class EncryptedTextField(BaseEncryptedField):
    __metaclass__ = models.SubfieldBase

    def get_internal_type(self):
        return 'TextField'

    def formfield(self, **kwargs):
        defaults = {'widget': forms.Textarea}
        defaults.update(kwargs)
        return super(EncryptedTextField, self).formfield(**defaults)

add_introspection_rules([
    (
        [EncryptedTextField], [], {},
    ),
], ["^django_common\.db_fields\.EncryptedTextField"])


class EncryptedCharField(BaseEncryptedField):
    __metaclass__ = models.SubfieldBase

    def get_internal_type(self):
        return "CharField"

    def formfield(self, **kwargs):
        defaults = {'max_length': self.max_length}
        defaults.update(kwargs)
        return super(EncryptedCharField, self).formfield(**defaults)

add_introspection_rules([
    (
        [EncryptedCharField], [], {},
    ),
], ["^django_common\.db_fields\.EncryptedCharField"])



import os

from django.conf import settings

from django.core.mail.backends.smtp import EmailBackend
from django.core.mail.backends.filebased import EmailBackend as FileEmailBackend
from django.core.mail import message


class TestEmailBackend(EmailBackend):
    """
        Email Backend to overwrite TO, CC and BCC in all outgoing emails to custom
        values.

        Sample values from setting.py:
        EMAIL_BACKEND = 'django_common.email_backends.TestEmailBackend'
        TEST_EMAIL_TO = ['dev@tivix.com']  # default are addresses form ADMINS
        TEST_EMAIL_CC = ['dev-cc@tivix.com']  # default is empty list
        TEST_EMAIL_BCC = ['dev-bcc@tivix.com']  # default is empty list
    """

    def _send(self, email_message):
        """A helper method that does the actual sending."""
        if not email_message.recipients():
            return False
        from_email = email_message.from_email
        if hasattr(message, 'sanitize_address'):
            from_email = message.sanitize_address(email_message.from_email,
                                                  email_message.encoding)
        if hasattr(settings, 'TEST_EMAIL_TO'):
            email_message.to = settings.TEST_EMAIL_TO
        else:
            email_message.to = dict(getattr(settings, 'ADMINS', ())).values()
        email_message.cc = getattr(settings, 'TEST_EMAIL_CC', [])
        email_message.bcc = getattr(settings, 'TEST_EMAIL_BCC', [])
        if hasattr(message, 'sanitize_address'):
            recipients = [message.sanitize_address(addr, email_message.encoding)
                          for addr in email_message.recipients()]
        else:
            recipients = email_message.recipients()
        try:
            self.connection.sendmail(from_email, recipients,
                                     email_message.message().as_string())
        except:
            if not self.fail_silently:
                raise
            return False
        return True


class CustomFileEmailBackend(FileEmailBackend):
    """
        Email Backend to save emails as file with custom extension. It makes easier
        to open emails in email applications, f.e. with eml extension for mozilla
        thunderbird.

        Sample values from setting.py:
        EMAIL_BACKEND = 'django_common.email_backends.CustomFileEmailBackend'
        EMAIL_FILE_PATH = '/email/file/path/'
        EMAIL_FILE_EXT = 'eml'
    """

    def _get_filename(self):
        filename = super(CustomFileEmailBackend, self)._get_filename()
        if hasattr(settings, 'EMAIL_FILE_EXT'):
            filename = '%s.%s' % (os.path.splitext(filename)[0], settings.EMAIL_FILE_EXT.strip('.'))
        return filename



from django import template
from django.forms import widgets
from django.template.loader import get_template
from django.template import Context

register = template.Library()


class FormFieldNode(template.Node):
    """
    Helper class for the render_form_field below
    """
    def __init__(self, form_field, help_text=None, css_classes=None):
        self.form_field = template.Variable(form_field)
        self.help_text = help_text[1:-1] if help_text else help_text
        self.css_classes = css_classes[1:-1] if css_classes else css_classes

    def render(self, context):
        try:
            form_field = self.form_field.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        widget = form_field.field.widget

        if isinstance(widget, widgets.RadioSelect):
            t = get_template('common/fragments/radio_field.html')
        elif isinstance(widget, widgets.CheckboxInput):
            t = get_template('common/fragments/checkbox_field.html')
        elif isinstance(widget, widgets.CheckboxSelectMultiple):
            t = get_template('common/fragments/multi_checkbox_field.html')
        else:
            t = get_template('common/fragments/form_field.html')

        if self.help_text is None:
            self.help_text = form_field.help_text

        return t.render(Context({
            'form_field': form_field,
            'help_text': self.help_text,
            'css_classes': self.css_classes
        }))


@register.tag
def render_form_field(parser, token):
    """
    Usage is {% render_form_field form.field_name optional_help_text optional_css_classes %}

    - optional_help_text and optional_css_classes are strings
    - if optional_help_text is not given, then it is taken from form field object
    """
    try:
        help_text = None
        css_classes = None

        token_split = token.split_contents()
        if len(token_split) == 4:
            tag_name, form_field, help_text, css_classes = token.split_contents()
        elif len(token_split) == 3:
            tag_name, form_field, help_text = token.split_contents()
        else:
            tag_name, form_field = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "Unable to parse arguments for %r" % token.contents.split()[0]

    return FormFieldNode(form_field, help_text=help_text, css_classes=css_classes)


@register.simple_tag
def active(request, pattern):
    """
    Returns the string 'active' if pattern matches. Used to assign a css class in navigation bars to active tab/section
    """
    if request.path == pattern:
        return 'active'
    return ''


@register.simple_tag
def active_starts(request, pattern):
    """
    Returns the string 'active' if request url starts with pattern. Used to assign a css class in navigation bars to
    active tab/section
    """
    if request.path.startswith(pattern):
        return 'active'
    return ''



from datetime import datetime, timedelta
from django.conf import settings

class SessionManagerBase(object):
    """
    Base class that a "SessionManager" concrete class should extend. It should have a list called _SESSION_KEYS that
    lists all the keys that class uses/depends on.
    
    Ideally each app has a session.py that has this class and is used in the apps views etc.
    """
    def __init__(self, request, prepend_key_with=''):
        self._session = request.session
        self._prepend_key_with = prepend_key_with
  
    def _get_or_set(self, key, value):
        key = '%s%s' % (self._prepend_key_with, key)
        
        if not value is None:
            self._session[key] = value
            return value
        return self._session.get(key)
  
    def reset_keys(self):
        for key in self._SESSION_KEYS:
            key = '%s%s' % (self._prepend_key_with, key)
            
            if self._session.has_key(key):
                del self._session[key]


class SessionManager(SessionManagerBase):
    """Manages storing the cart"""

    USER_ONLINE_TIMEOUT = 180  # 3 min

    USERTIME = 'usertime'
    _GENERIC_VAR_KEY_PREFIX = 'lpvar_'   # handles generic stuff being stored in the session

    _SESSION_KEYS = [
        USERTIME,
    ]

    def __init__(self, request):
        super(SessionManager, self).__init__(request, prepend_key_with=request.get_host())
        if not self._get_or_set(self.USERTIME, None):
            self._get_or_set(self.USERTIME, None)

    def get_usertime(self):
        usertime = self._get_or_set(self.USERTIME, None)
        try:
            return usertime['last'] - usertime['start']
        except:
            return 0

    def ping_usertime(self):
        # Override default user online timeout
        try:
            timeout = int(settings.USER_ONLINE_TIMEOUT)
        except:
            timeout = self.USER_ONLINE_TIMEOUT
        if not self._get_or_set(self.USERTIME, None):
            self._get_or_set(self.USERTIME, {'start': datetime.now(), 'last': datetime.now()})
        else:
            usertime = self._get_or_set(self.USERTIME, None)
            if usertime['last'] + timedelta(seconds=timeout) < datetime.now():
                # This mean user reached timeout - we start from begining
                self._get_or_set(self.USERTIME, {'start': datetime.now(), 'last': datetime.now()})
            else:
                # We just update last time
                usertime['last'] = datetime.now()
        return self._get_or_set(self.USERTIME, None)

    def clear_usertime(self):
        return self._get_or_set(self.USERTIME, {})

    def generic_var(self, key, value=None):
        """Stores generic variables in the session prepending it with
        _GENERIC_VAR_KEY_PREFIX."""
        return self._get_or_set('%s%s' % (self._GENERIC_VAR_KEY_PREFIX, key), value)



import logging

from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User


class EmailBackend(ModelBackend):
    def authenticate(self, username=None, password=None):
        """
        "username" being passed is really email address and being compared to as such.
        """
        try:
            user = User.objects.get(email=username)
            if user.check_password(password):
                return user
        except (User.DoesNotExist, User.MultipleObjectsReturned):
            logging.warn('Unsuccessful login attempt using username/email: %s' % username)
        
        return None



"Some common routines that can be used throughout the code."
import hashlib, os, logging, datetime, threading

from django.utils import simplejson
from django.utils.encoding import force_unicode
from django.template import Context
from django.template.loader import get_template
from django.core import exceptions

from django_common.tzinfo import utc, Pacific


class AppException(exceptions.ValidationError):
    """Base class for exceptions used in our system.

    A common base class permits application code to distinguish between exceptions raised in our code from ones raised
    in libraries.
    """
    pass


class InvalidContentType(AppException):
    def __init__(self, file_types, msg=None):
        msg = msg or 'Only the following file content types are permitted: %s' % str(file_types)
        super(self.__class__, self).__init__(msg)
        self.file_types = file_types


class FileTooLarge(AppException):
    def __init__(self, file_size_kb, msg=None):
        msg = msg or 'Files may not be larger than %s KB' % file_size_kb
        super(self.__class__, self).__init__(msg)
        self.file_size = file_size_kb


def get_class(kls):
    """
    Converts a string to a class.
    Courtesy: http://stackoverflow.com/questions/452969/does-python-have-an-equivalent-to-java-class-forname/452981#452981
    """
    parts = kls.split('.')
    module = ".".join(parts[:-1])
    m = __import__(module)
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m


def is_among(value, *possibilities):
    """Ensure that the method that has been used for the request is one of the expected ones (e.g., GET or POST)."""
    for possibility in possibilities:
        if value == possibility:
            return True
    raise Exception, 'A different request value was encountered than expected: %s' % value


def form_errors_serialize(form):
    errors = {}
    for field in form.fields.keys():
        if form.errors.has_key(field):
            if form.prefix:
                errors['%s-%s' % (form.prefix, field)] = force_unicode(form.errors[field])
            else:
                errors[field] = force_unicode(form.errors[field])

    if form.non_field_errors():
        errors['non_field_errors'] = force_unicode(form.non_field_errors())
    return {'errors': errors}


def json_response(data={}, errors=[], success=True):
    data.update({
        'errors': errors,
        'success': len(errors) == 0 and success,
    })
    return simplejson.dumps(data)


def sha224_hash():
    return hashlib.sha224(os.urandom(224)).hexdigest()


def sha1_hash():
    return hashlib.sha1(os.urandom(224)).hexdigest()


def md5_hash(image=None, max_length=None):
    # TODO:  Figure out how much entropy is actually needed, and reduce the current number of bytes if possible if doing
    # so will result in a performance improvement.
    if max_length:
        assert max_length > 0

    ret = hashlib.md5(image or os.urandom(224)).hexdigest()
    return ret if not max_length else ret[:max_length]


def start_thread(target, *args):
    t = threading.Thread(target=target, args=args)
    t.setDaemon(True)
    t.start()


def send_mail(subject, message, from_email, recipient_emails, files=None,
        html=False, reply_to=None, bcc=None, cc=None, files_manually=None):
    """Sends email with advanced optional parameters

    To attach non-file content (e.g. content not saved on disk), use
    files_manually parameter and provide list of 3 element tuples, e.g.
    [('design.png', img_data, 'image/png'),] which will be passed to
    email.attach().
    """
    import django.core.mail
    try:
        logging.debug('Sending mail to: %s' % ', '.join(r for r in recipient_emails))
        logging.debug('Message: %s' % message)
        email = django.core.mail.EmailMessage(subject, message, from_email, recipient_emails, bcc, cc=cc)
        if html:
            email.content_subtype = "html"
        if files:
            for file in files:
                email.attach_file(file)
        if files_manually:
            for filename, content, mimetype in files_manually:
                email.attach(filename, content, mimetype)
        if reply_to:
            email.extra_headers = {'Reply-To': reply_to}
        email.send()
    except Exception, e:
        # TODO:  Raise error again so that more information is included in the logs?
        logging.error('Error sending message [%s] from %s to %s %s' % (subject, from_email, recipient_emails, e))


def send_mail_in_thread(subject, message, from_email, recipient_emails, files=None, html=False, reply_to=None, bcc=None, cc=None, files_manually=None):
    start_thread(send_mail, subject, message, from_email, recipient_emails, files, html, reply_to, bcc, cc, files_manually)


def send_mail_using_template(subject, template_name, from_email, recipient_emails, context_map, in_thread=False, files=None, html=False, reply_to=None, bcc=None, cc=None, files_manually=None):
    t = get_template(template_name)
    message = t.render(Context(context_map))
    if in_thread:
        return send_mail_in_thread(subject, message, from_email, recipient_emails, files, html, reply_to, bcc, cc, files_manually)
    else:
        return send_mail(subject, message, from_email, recipient_emails, files, html, reply_to, bcc, cc, files_manually)


def utc_to_pacific(timestamp):
    return timestamp.replace(tzinfo=utc).astimezone(Pacific)


def pacific_to_utc(timestamp):
    return timestamp.replace(tzinfo=Pacific).astimezone(utc)


def humanize_time_since(timestamp = None):
    """Returns a fuzzy time since. Will only return the largest time. EX: 20 days, 14 min"""

    timeDiff = datetime.datetime.now() - timestamp
    days = timeDiff.days
    hours = timeDiff.seconds/3600
    minutes = timeDiff.seconds%3600/60
    seconds = timeDiff.seconds%3600%60

    str = ""
    tStr = ""
    if days > 0:
        if days == 1:   tStr = "day"
        else:           tStr = "days"
        str = str + "%s %s" %(days, tStr)
        return str
    elif hours > 0:
        if hours == 1:  tStr = "hour"
        else:           tStr = "hours"
        str = str + "%s %s" %(hours, tStr)
        return str
    elif minutes > 0:
        if minutes == 1:tStr = "min"
        else:           tStr = "mins"
        str = str + "%s %s" %(minutes, tStr)
        return str
    elif seconds > 0:
        if seconds == 1:tStr = "sec"
        else:           tStr = "secs"
        str = str + "%s %s" %(seconds, tStr)
        return str
    else:
        return str


def chunks(l, n):
    """ split successive n-sized chunks from a list."""
    for i in xrange(0, len(l), n):
        yield l[i:i+n]



#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
    from setuptools.command.test import test
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages
    from setuptools.command.test import test

from django_common import settings

import os

here = os.path.dirname(os.path.abspath(__file__))
f = open(os.path.join(here, 'README.rst'))
long_description = f.read().strip()
f.close()

setup(
    name='django-common-helpers',
    version=settings.VERSION,
    author='Tivix',
    author_email='dev@tivix.com',
    url='http://github.com/tivix/django-common',
    description = 'Common things every Django app needs!',
    packages=find_packages(),
    long_description=long_description,
    keywords = 'django',
    zip_safe=False,
    install_requires=[
        'Django>=1.4.0',
        'South>=0.7.2'
    ],
    test_suite = 'django_common.tests',
    include_package_data=True,
    # cmdclass={},
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)



from StringIO import StringIO

from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponsePermanentRedirect, HttpResponseRedirect, Http404
from django.utils import simplejson


class JsonResponse(HttpResponse):
  def __init__(self, data={ }, errors=[ ], success=True):
    """
    data is a map, errors a list
    """
    json = json_response(data=data, errors=errors, success=success)
    super(JsonResponse, self).__init__(json, mimetype='application/json')

class JsonpResponse(HttpResponse):
  """
  Padded JSON response, used for widget XSS
  """
  def __init__(self, request, data={ }, errors=[ ], success=True):
    """
    data is a map, errors a list
    """
    json = json_response(data=data, errors=errors, success=success)
    js = "%s(%s)" % (request.GET.get("jsonp", "jsonp_callback"), json)
    super(JsonpResponse, self).__init__(js, mimetype='application/javascipt')

def json_response(data={ }, errors=[ ], success=True):
  data.update({
    'errors': errors,
    'success': len(errors) == 0 and success,
  })
  return simplejson.dumps(data)


class XMLResponse(HttpResponse):
  def __init__(self, data):
    """
    data is the entire xml body/document
    """
    super(XMLResponse, self).__init__(data, mimetype='text/xml')



from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

class LoginRequiredMixin(object):
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)


from django.db import transaction, models
from django.views.decorators.csrf import csrf_protect
from django.utils.decorators import method_decorator
from django.contrib.admin.util import unquote, flatten_fieldsets
from django.contrib.admin.options import BaseModelAdmin, ModelAdmin
from django.contrib.admin.helpers import AdminForm
from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.utils.html import escape
from django.forms.formsets import all_valid
from django.contrib.admin import helpers
from django.utils.safestring import mark_safe
from django.forms.models import (inlineformset_factory, BaseInlineFormSet)
from django import forms
from django.utils.functional import curry

csrf_protect_m = method_decorator(csrf_protect)


def __init__(self, form, fieldsets, prepopulated_fields, readonly_fields=None, model_admin=None):
    """
    Monkey-patch for django 1.5
    """
    def normalize_fieldsets(fieldsets):
        """
        Make sure the keys in fieldset dictionaries are strings. Returns the
        normalized data.
        """
        result = []
        for name, options in fieldsets:
            result.append((name, normalize_dictionary(options)))
        return result

    def normalize_dictionary(data_dict):
        """
        Converts all the keys in "data_dict" to strings. The keys must be
        convertible using str().
        """
        for key, value in data_dict.items():
            if not isinstance(key, str):
                del data_dict[key]
                data_dict[str(key)] = value
        return data_dict
    if isinstance(prepopulated_fields, list):
        prepopulated_fields = dict()
    self.form, self.fieldsets = form, normalize_fieldsets(fieldsets)
    self.prepopulated_fields = [{
        'field': form[field_name],
        'dependencies': [form[f] for f in dependencies]
    } for field_name, dependencies in prepopulated_fields.items()]
    self.model_admin = model_admin
    if readonly_fields is None:
        readonly_fields = ()
    self.readonly_fields = readonly_fields

AdminForm.__init__ = __init__


class NestedModelAdmin(ModelAdmin):

    @csrf_protect_m
    @transaction.commit_on_success
    def add_view(self, request, form_url='', extra_context=None):
        "The 'add' admin view for this model."
        model = self.model
        opts = model._meta

        if not self.has_add_permission(request):
            raise PermissionDenied
        ModelForm = self.get_form(request)
        formsets = []
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES)
            if form.is_valid():
                new_object = self.save_form(request, form, change=False)
                form_validated = True
            else:
                form_validated = False
                new_object = self.model()
            prefixes = {}
            for FormSet, inline in zip(self.get_formsets(request), self.get_inline_instances(request)):
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                formset = FormSet(data=request.POST, files=request.FILES,
                                  instance=new_object,
                                  save_as_new="_saveasnew" in request.POST,
                                  prefix=prefix, queryset=inline.queryset(request))
                formsets.append(formset)
                for inline in self.get_inline_instances(request):
                    # If this is the inline that matches this formset, and
                    # we have some nested inlines to deal with, then we need
                    # to get the relevant formset for each of the forms in
                    # the current formset.
                    if inline.inlines and inline.model == formset.model:
                        for nested in inline.inline_instances:
                            for the_form in formset.forms:
                                InlineFormSet = nested.get_formset(request, the_form.instance)
                                prefix = "%s-%s" % (the_form.prefix, InlineFormSet.get_default_prefix())
                                formsets.append(InlineFormSet(request.POST, request.FILES, instance=the_form.instance, prefix=prefix))
            if all_valid(formsets) and form_validated:
                self.save_model(request, new_object, form, change=False)
                form.save_m2m()
                for formset in formsets:
                    self.save_formset(request, form, formset, change=False)

                self.log_addition(request, new_object)
                return self.response_add(request, new_object)
        else:
            # Prepare the dict of initial data from the request.
            # We have to special-case M2Ms as a list of comma-separated PKs.
            initial = dict(request.GET.items())
            for k in initial:
                try:
                    f = opts.get_field(k)
                except models.FieldDoesNotExist:
                    continue
                if isinstance(f, models.ManyToManyField):
                    initial[k] = initial[k].split(",")
            form = ModelForm(initial=initial)
            prefixes = {}
            for FormSet, inline in zip(self.get_formsets(request),
                                       self.get_inline_instances(request)):
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                formset = FormSet(instance=self.model(), prefix=prefix,
                                  queryset=inline.queryset(request))
                formsets.append(formset)

        adminForm = helpers.AdminForm(form, list(self.get_fieldsets(request)),
            self.prepopulated_fields, self.get_readonly_fields(request),
            model_admin=self)
        media = self.media + adminForm.media

        inline_admin_formsets = []
        for inline, formset in zip(self.get_inline_instances(request), formsets):
            fieldsets = list(inline.get_fieldsets(request))
            readonly = list(inline.get_readonly_fields(request))
            inline_admin_formset = helpers.InlineAdminFormSet(inline, formset,
                fieldsets, readonly, model_admin=self)
            if inline.inlines:
                for form in formset.forms:
                    if form.instance.pk:
                        instance = form.instance
                    else:
                        instance = None
                    form.inlines = inline.get_inlines(request, instance, prefix=form.prefix)
                inline_admin_formset.inlines = inline.get_inlines(request)
            inline_admin_formsets.append(inline_admin_formset)
            media = media + inline_admin_formset.media

        context = {
            'title': _('Add %s') % force_unicode(opts.verbose_name),
            'adminform': adminForm,
            'is_popup': "_popup" in request.REQUEST,
            'show_delete': False,
            'media': mark_safe(media),
            'inline_admin_formsets': inline_admin_formsets,
            'errors': helpers.AdminErrorList(form, formsets),
            'app_label': opts.app_label,
        }
        context.update(extra_context or {})
        return self.render_change_form(request, context, form_url=form_url, add=True)

    @csrf_protect_m
    @transaction.commit_on_success
    def change_view(self, request, object_id, extra_context=None):
        "The 'change' admin view for this model."
        model = self.model
        opts = model._meta
        obj = self.get_object(request, unquote(object_id))

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})

        if request.method == 'POST' and "_saveasnew" in request.POST:
            return self.add_view(request, form_url='../add/')

        ModelForm = self.get_form(request, obj)
        formsets = []
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES, instance=obj)
            if form.is_valid():
                form_validated = True
                new_object = self.save_form(request, form, change=True)
            else:
                form_validated = False
                new_object = obj
            prefixes = {}
            for FormSet, inline in zip(self.get_formsets(request, new_object),
                                       self.get_inline_instances(request)):
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                formset = FormSet(request.POST, request.FILES,
                                  instance=new_object, prefix=prefix,
                                  queryset=inline.queryset(request))

                formsets.append(formset)
                for inline in self.get_inline_instances(request):
                    # If this is the inline that matches this formset, and
                    # we have some nested inlines to deal with, then we need
                    # to get the relevant formset for each of the forms in
                    # the current formset.
                    if inline.inlines and inline.model == formset.model:
                        for nested in inline.inline_instances:
                            for the_form in formset.forms:
                                InlineFormSet = nested.get_formset(request, the_form.instance)
                                prefix = "%s-%s" % (the_form.prefix, InlineFormSet.get_default_prefix())
                                formsets.append(InlineFormSet(request.POST, request.FILES, instance=the_form.instance, prefix=prefix))
            if all_valid(formsets) and form_validated:
                self.save_model(request, new_object, form, change=True)
                form.save_m2m()
                for formset in formsets:
                    self.save_formset(request, form, formset, change=True)

                change_message = self.construct_change_message(request, form, formsets)
                self.log_change(request, new_object, change_message)
                return self.response_change(request, new_object)

        else:
            form = ModelForm(instance=obj)
            prefixes = {}
            for FormSet, inline in zip(self.get_formsets(request, obj), self.get_inline_instances(request)):
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                formset = FormSet(instance=obj, prefix=prefix,
                                  queryset=inline.queryset(request))
                formsets.append(formset)

        adminForm = helpers.AdminForm(form, self.get_fieldsets(request, obj),
            self.prepopulated_fields, self.get_readonly_fields(request, obj),
            model_admin=self)
        media = self.media + adminForm.media

        inline_admin_formsets = []
        for inline, formset in zip(self.get_inline_instances(request), formsets):
            fieldsets = list(inline.get_fieldsets(request, obj))
            readonly = list(inline.get_readonly_fields(request, obj))
            inline_admin_formset = helpers.InlineAdminFormSet(inline, formset,
                fieldsets, readonly, model_admin=self)
            if inline.inlines:
                for form in formset.forms:
                    if form.instance.pk:
                        instance = form.instance
                    else:
                        instance = None
                    form.inlines = inline.get_inlines(request, instance, prefix=form.prefix)
                inline_admin_formset.inlines = inline.get_inlines(request)
            inline_admin_formsets.append(inline_admin_formset)
            media = media + inline_admin_formset.media

        context = {
            'title': _('Change %s') % force_unicode(opts.verbose_name),
            'adminform': adminForm,
            'object_id': object_id,
            'original': obj,
            'is_popup': "_popup" in request.REQUEST,
            'media': mark_safe(media),
            'inline_admin_formsets': inline_admin_formsets,
            'errors': helpers.AdminErrorList(form, formsets),
            'app_label': opts.app_label,
        }
        context.update(extra_context or {})
        return self.render_change_form(request, context, change=True, obj=obj)

    def get_inlines(self, request, obj=None, prefix=None):
        nested_inlines = []
        for inline in self.get_inline_instances(request):
            FormSet = inline.get_formset(request, obj)
            prefix = "%s-%s" % (prefix, FormSet.get_default_prefix())
            formset = FormSet(instance=obj, prefix=prefix)
            fieldsets = list(inline.get_fieldsets(request, obj))
            nested_inline = helpers.InlineAdminFormSet(inline, formset, fieldsets)
            nested_inlines.append(nested_inline)
        return nested_inlines


class NestedTabularInline(BaseModelAdmin):
    """
    Options for inline editing of ``model`` instances.

    Provide ``name`` to specify the attribute name of the ``ForeignKey`` from
    ``model`` to its parent. This is required if ``model`` has more than one
    ``ForeignKey`` to its parent.
    """
    model = None
    fk_name = None
    formset = BaseInlineFormSet
    extra = 3
    max_num = None
    template = None
    verbose_name = None
    verbose_name_plural = None
    can_delete = True
    template = 'common/admin/nested_tabular.html'
    inlines = []

    def __init__(self, parent_model, admin_site):
        self.admin_site = admin_site
        self.parent_model = parent_model
        self.opts = self.model._meta
        super(NestedTabularInline, self).__init__()
        if self.verbose_name is None:
            self.verbose_name = self.model._meta.verbose_name
        if self.verbose_name_plural is None:
            self.verbose_name_plural = self.model._meta.verbose_name_plural
        self.inline_instances = []
        for inline_class in self.inlines:
            inline_instance = inline_class(self.model, self.admin_site)
            self.inline_instances.append(inline_instance)

    def _media(self):
        from django.conf import settings
        js = ['js/jquery.min.js', 'js/jquery.init.js', 'js/inlines.min.js']
        if self.prepopulated_fields:
            js.append('js/urlify.js')
            js.append('js/prepopulate.min.js')
        if self.filter_vertical or self.filter_horizontal:
            js.extend(['js/SelectBox.js', 'js/SelectFilter2.js'])
        return forms.Media(js=['%s%s' % (settings.ADMIN_MEDIA_PREFIX, url) for url in js])
    media = property(_media)

    def get_formset(self, request, obj=None, **kwargs):
        """Returns a BaseInlineFormSet class for use in admin add/change views."""
        if self.declared_fieldsets:
            fields = flatten_fieldsets(self.declared_fieldsets)
        else:
            fields = None
        if self.exclude is None:
            exclude = []
        else:
            exclude = list(self.exclude)
        exclude.extend(kwargs.get("exclude", []))
        exclude.extend(self.get_readonly_fields(request, obj))
        # if exclude is an empty list we use None, since that's the actual
        # default
        exclude = exclude or None
        defaults = {
            "form": self.form,
            "formset": self.formset,
            "fk_name": self.fk_name,
            "fields": fields,
            "exclude": exclude,
            "formfield_callback": curry(self.formfield_for_dbfield, request=request),
            "extra": self.extra,
            "max_num": self.max_num,
            "can_delete": self.can_delete,
        }
        defaults.update(kwargs)
        return inlineformset_factory(self.parent_model, self.model, **defaults)

    def get_fieldsets(self, request, obj=None):
        if self.declared_fieldsets:
            return self.declared_fieldsets
        form = self.get_formset(request).form
        fields = form.base_fields.keys() + list(self.get_readonly_fields(request, obj))
        return [(None, {'fields': fields})]

    def get_inlines(self, request, obj=None, prefix=None):
        nested_inlines = []
        for inline in self.inline_instances:
            FormSet = inline.get_formset(request, obj)
            prefix = "%s-%s" % (prefix, FormSet.get_default_prefix())
            formset = FormSet(instance=obj, prefix=prefix)
            fieldsets = list(inline.get_fieldsets(request, obj))
            nested_inline = helpers.InlineAdminFormSet(inline, formset, fieldsets)
            nested_inlines.append(nested_inline)
        return nested_inlines
